@@ -357,6 +357,7 @@ int devlist_ready = 0;
 #ifdef UCLIBC
 int add_dev(const char *name, const struct stat *stb, int flag, struct FTW *s)
 {
+	return 0;
 }
 char *map_dev(int major, int minor, int create)
 {
@@ -375,7 +376,7 @@ int add_dev_1(const char *name, const struct stat *stb, int flag)
 }
 int nftw(const char *path, int (*han)(const char *name, const struct stat *stb, int flag, struct FTW *s), int nopenfd, int flags)
 {
-	ftw(path, add_dev_1, nopenfd);
+	return ftw(path, add_dev_1, nopenfd);
 }
 #endif
 
@@ -415,10 +416,23 @@ char *map_dev(int major, int minor, int create)
 	struct devmap *p;
 	char *std = NULL, *nonstd=NULL;
 	int did_check = 0;
+
+	if (major == 0 && minor == 0) {
+		if (!create)
+			return NULL;
+		else
+			return "0:0";
+	}
  retry:
 	if (!devlist_ready) {
 		char *dev = "/dev";
 		struct stat stb;
+		while(devlist) {
+			struct devmap *d = devlist;
+			devlist = d->next;
+			free(d->name);
+			free(d);
+		}
 		if (lstat(dev, &stb)==0 &&
 		    S_ISLNK(stb.st_mode))
 			dev = "/dev/.";
@@ -446,7 +460,7 @@ char *map_dev(int major, int minor, int create)
 	}
 	if (create && !std && !nonstd) {
 		static char buf[30];
-		snprintf(buf, 1024, "%d:%d", major, minor);
+		snprintf(buf, sizeof(buf), "%d:%d", major, minor);
 		nonstd = buf;
 	}
 
